import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.Map;

import jodd.json.JsonParser;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.TextField;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.index.LeafReader;
import org.apache.lucene.index.Terms;
import org.apache.lucene.index.TermsEnum;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.queryparser.simple.SimpleQueryParser;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;

import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;



public class IndexTester {

	public static void main(String[] args) throws IOException, ParseException {
		
        if (args.length != 3) {
            System.err.println("Incorrect number of arguments! Usage:");
            System.err.println("");
            System.err.println("java IndexTester should_clear_index path_to_data path_to_index ");
            System.err.println("\tif should_clear_index is \"1\", the index will be rebuilt. Otherwise, it will try and use an existing index.");
            System.err.println("\tpath_to_index should point to an empty directory somewhere.");
            System.exit(-1);
        }
		
    String shouldClearIndex = args[0];
		String inputPath = args[1]; // where to find the file containing the JSON to index
		String idxDirPath = args[2]; // where to put/find the Lucene index we want to search
        
        
		File inputFile = new File(inputPath);
		
		// set up our analyzer:
		StandardAnalyzer analyzer = new StandardAnalyzer();
		
		// set up the index
		File idxDir = new File(idxDirPath);
	
		Directory dir = FSDirectory.open(idxDir.toPath());
		
    // are we clearing and rebuilding the index?
    if (shouldClearIndex.compareTo("1") == 0) {
        System.out.println("Rebuilding index...");
    
        IndexWriterConfig idxConfig = new IndexWriterConfig(analyzer);
        idxConfig.setOpenMode(IndexWriterConfig.OpenMode.CREATE);
        IndexWriter idxWriter = new IndexWriter(dir, idxConfig);

        // Now, populate the index:
        int idx = 0;
        JsonParser jParser = new JsonParser();
        
        for (String line : Files.readAllLines(inputFile.toPath(), StandardCharsets.UTF_8)) {
            // On large amounts of data, this can take a while- let's print some status updates
            if (idx % 1000 == 0) {
                System.out.println(idx);
            }
            idx++;
            
            // each line of the input file is a serialized JSON object
            Map j = jParser.parse(line);
            // simple types (strings, numbers, etc.) are handled like so:
            String title = (String)j.get("proj_title");
            // complex types (lists or dicts) get turned into instances of
            // java.util.Map and java.util.List.
            
            // Among other fields, the grants all have abstracts:
            String ab = (String)j.get("abstract_text");
            
            // There are lots of other fields in the JSON file....

            // Look at the docs for TextField to see about other types- Lucene can index numbers, dates, etc.
            Field tiField = new Field("title", title, TextField.TYPE_STORED); 
            // The TYPE_STORED directive tells Lucene to actually store the original token in the index. This is handy 
            // for all sorts of reasons! (term highlighting, etc.)
            
            // Make a new Document instance...
            Document thisDoc = new Document();

            // set up its fields...
            thisDoc.add(tiField);

            // add our document to the index
            idxWriter.addDocument(thisDoc);

        }

        System.out.println("Done!");

        System.out.println(idx + " documents indexed.");

        idxWriter.close();

    }

		
		// Open up the index for querying:
		DirectoryReader reader = DirectoryReader.open(dir);

		
		// Now search
		IndexSearcher searcher = new IndexSearcher(reader);
		
		// Things to note re: QueryParser:
		// 1. 	The first argument is the "default" field to search- 
		//		if nothing else is specified, in the query, this is what
		//		will be searched.
		// 2.	You always want to make sure to use the same Analyzer for your
		//		query as you did when you built the index!
    //
    // Other query parser classes will behave similarly, but may have different argument ordering.
        
		QueryParser qParser = new QueryParser("title", analyzer); 
		

    // NOTE: for demo and debugginer purposes, I have it set up to read a query as input from STDIN
    // For the assignment, you'll want to build your queries programmatically.
		System.out.print("Query: ");
        String queryText = System.console().readLine();
		
		if (queryText.compareTo("") != 0) {
		
			Query q = qParser.parse(queryText);
			TopDocs results = searcher.search(q,  100);
			System.out.println("Got " + results.totalHits + " hits!");
			for (ScoreDoc d : results.scoreDocs) {
				System.out.println(d.doc + "\t" + d.score);
				Document res = reader.document(d.doc);
				System.out.println(res.getField("title").stringValue());
			}
		}
		
	}

}
